﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading;
using System.Xml;
using Sasa.QualityTools.CodeAnalysis.Common;

namespace Sasa.QualityTools.CodeAnalysis.FxCopIntegrator.Models
{
    public class FxCopReport
    {
        public ICollection<Problem> Problems { get; private set; }

        public ICollection<Rule> Rules { get; private set; }



        private FxCopReport()
        {
            Problems = new List<Problem>();
            Rules = new List<Rule>();
        }


        public static FxCopReport Load(string path)
        {
            Logger.Info("Loading FxCop report from '{0}'.", path);

            if (!File.Exists(path)) throw new InvalidOperationException("The specified file (" + path + ") doesn't exist.");

            XmlDocument document = new XmlDocument();
            try
            {
                document.Load(path);
            }
            catch (Exception e)
            {
                // TODO: change exception type.
                throw new CodeAnalysisException("Failed to load the specified FxCop report (" + path + ").", e);
            }

            var report = new FxCopReport();

            foreach (XmlNode node in document.SelectNodes("//Message"))
            {
                foreach (Problem problem in GetProblemsFrom(node))
                {
                    report.Problems.Add(problem);
                }
            }
            foreach (XmlNode node in document.SelectNodes("/FxCopReport/Rules/Rule"))
            {
                report.Rules.Add(GetRuleFrom(node));
            }

            return report;
        }

        internal static List<Problem> GetProblemsFrom(XmlNode messageNode)
        {
            var culture = Thread.CurrentThread.CurrentUICulture;

            var template = new Problem();
            template.CheckId = messageNode.Attributes["CheckId"].Value;
            template.Category = messageNode.Attributes["Category"].Value;
            template.Id = messageNode.SelectSingleNode("../../@Name").Value;
            template.Kind = GetCodeKind(messageNode);

            if (IsType(template.Kind))
            {
                string ns = messageNode.SelectSingleNode("ancestor::Namespace/@Name").Value;
                template.TypeName = String.Format(culture, "{0}.{1}", ns, template.Id);
            }
            else if (IsMember(template.Kind))
            {
                string ns = messageNode.SelectSingleNode("ancestor::Namespace/@Name").Value;
                string type = messageNode.SelectSingleNode("ancestor::Type/@Name").Value;
                template.TypeName = String.Format(culture, "{0}.{1}", ns, type);
            }

            XmlNode targetNameNode = messageNode.SelectSingleNode("ancestor::Target/@Name");
            if (targetNameNode != null)
            {
                // TODO: convert
                //var assemblyPath = new StringBuilder(targetNameNode.Value);
                //assemblyPath.Replace("$(ProjectDir)", "");
                template.AssemblyPath = targetNameNode.Value;
            }

            List<Problem> problems = new List<Problem>();
            foreach (XmlNode node in messageNode.SelectNodes("Issue"))
            {
                Problem problem = template.Copy();
                if (node.Attributes["Path"] != null && node.Attributes["File"] != null)
                {
                    problem.Path = Path.Combine(node.Attributes["Path"].Value, node.Attributes["File"].Value);
                }
                if (node.Attributes["Line"] != null)
                {
                    problem.Line = Int32.Parse(node.Attributes["Line"].Value, culture);
                }
                problem.Message = node.InnerText;
                problems.Add(problem);
            }

            return problems;
        }

        private static CodeKind GetCodeKind(XmlNode messageNode)
        {
            CodeKind kind = CodeKind.Unknown;

            XmlNode kindAttr = messageNode.SelectSingleNode("../../@Kind");
            if (kindAttr != null)
            {
                Enum.TryParse<CodeKind>(kindAttr.Value, true, out kind);
            }
            else
            {
                Enum.TryParse<CodeKind>(messageNode.SelectSingleNode("../..").Name, true, out kind);
            }

            return kind;
        }

        private static bool IsType(CodeKind kind)
        {
            return kind == CodeKind.Class || kind == CodeKind.Delegate || kind == CodeKind.Enum || kind == CodeKind.Interface || kind == CodeKind.Struct;
        }

        private static bool IsMember(CodeKind kind)
        {
            return kind == CodeKind.Constructor || kind == CodeKind.Event || kind == CodeKind.Field || kind == CodeKind.Method || kind == CodeKind.Property;
        }

        /// <summary>
        /// Get the rule metadata from the specified Rule node of the FxCop report.
        /// </summary>
        /// <param name="ruleNode">The Rule node of the FxCop report.</param>
        /// <returns>The rule metadata.</returns>
        internal static Rule GetRuleFrom(XmlNode ruleNode)
        {
            Rule rule = new Rule();

            rule.CheckId = ruleNode.Attributes["CheckId"].Value;
            rule.Category = ruleNode.Attributes["Category"].Value;
            rule.Name = ruleNode["Name"].InnerText;
            rule.Description = ruleNode["Description"].InnerText;

            return rule;
        }
    }
}
