﻿using Microsoft.Build.Framework;
using Microsoft.Win32;
using Sasa.QualityTools.CodeAnalysis.Common;
using Sasa.QualityTools.CodeAnalysis.Core.Models;
using Sasa.QualityTools.CodeAnalysis.Core.Utilities;
using System;
using System.Globalization;
using System.IO;
using System.Reflection;

namespace Sasa.QualityTools.CodeAnalysis.MSBuild.Tasks
{
    public class PerformCodeInspection : PerformCodeAnalysisTaskBase
    {
        private const string RegistryKeyName = @"HKEY_CURRENT_USER\Software\Microsoft\VisualStudio\10.0\DialogPage\Sasa.QualityTools.CodeAnalysis.FxCopIntegrator.Options.GeneralOptionPageData";

        private const string RegistryValueName = "FxCopCmdPath";


        public string FxCopCmdPath { get; set; }


        protected override bool PerformCodeAnalysis()
        {
            if (FxCopCmdPath == null)
            {
                var registryValue = Registry.GetValue(RegistryKeyName, RegistryValueName, null);
                if (registryValue == null)
                {
                    var message = Text.Format("The FxCopCmd.exe path can't be retrieved from the registry ({0}\\{1}).", RegistryKeyName, RegistryValueName);
                    return false;
                }
                FxCopCmdPath = Convert.ToString(registryValue, CultureInfo.InvariantCulture);
            }

            if (!File.Exists(FxCopCmdPath))
            {
                var message = Text.Format("The specified FxCopCmd.exe path ({0}) doesn't exist.", FxCopCmdPath);
                return false;
            }

            var version = AssemblyName.GetAssemblyName(FxCopCmdPath).Version;
            var option = FxCopCmdOptionBuilder.Build(Project, version);

            var inspector = new CodeInspector();
            inspector.CommandPath = FxCopCmdPath;
            inspector.CommandOption = option;
            foreach (var r in References)
            {
                inspector.ReferenceAssemblyPaths.Add(r);
            }

            var result = FxCopErrorCodes.None;
            try
            {
                result = inspector.PerformCodeAnalysis();
            }
            catch (CodeAnalysisException e)
            {
                foreach (var log in ErrorLogs)
                {
                    Log.LogError(log);
                }
                Log.LogErrorFromException(e);
                return false;
            }

            foreach (var problem in inspector.Report.Problems)
            {
                WriteWarning(problem.Path, problem.Line, problem.CheckId, problem.Category, problem.Message);
            }
            foreach (var error in inspector.Report.Errors)
            {
                Log.LogError(error.Message);
            }

            if (result == FxCopErrorCodes.BuildBreakingMessage)
            {
                return false;
            }

            return true;
        }

        private void WriteWarning(string path, int? line, string checkId, string category, string message)
        {
            var lineNumber = (line.HasValue) ? line.Value : 1;
            var location = (!String.IsNullOrWhiteSpace(path)) ? String.Format("{0}({1},1)", path, lineNumber) : "[Location not stored in Pdb]";
            LogEventsFromTextOutput(Text.Format("{0} : warning : {1} : {2} : {3}", location, checkId, category, message), MessageImportance.Normal);
        }
    }
}
