﻿using Microsoft.Build.Utilities;
using Sasa.QualityTools.CodeAnalysis.Core.Models;
using Sasa.QualityTools.CodeAnalysis.MSBuild.Loggers;
using Sasa.QualityTools.CodeAnalysis.MSBuild.VisualStudio;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Sasa.QualityTools.CodeAnalysis.Core.VisualStudio;

namespace Sasa.QualityTools.CodeAnalysis.MSBuild.Tasks
{
    public abstract class PerformCodeAnalysisTaskBase : ToolTask
    {
        private MSBuildLogger logger;


        public string TargetPath { get; set; }

        public string SolutionDir { get; set; }

        public string ProjectDir { get; set; }

        public string[] References { get; set; }

        protected ISolution Solution { get; private set; }

        protected IProject Project { get; private set; }


        protected IEnumerable<string> ErrorLogs
        {
            get
            {
                return (logger != null) ? logger.Errors : new string[] { };
            }
        }


        protected PerformCodeAnalysisTaskBase()
        {
        }


        protected abstract bool PerformCodeAnalysis();

        public override bool Execute()
        {
            if (!CanExecute())
            {
                return false;
            }

            Solution = new VSSolutionEmulator(SolutionDir);
            Project = new VSProjectEmulator(Solution, TargetPath, ProjectDir);

            var config = CodeAnalysisConfiguration.Load(Project.ConfigurationPath);
            config.Override(CodeAnalysisConfiguration.Load(Solution.ConfigurationPath));
            if (!config.EnableCodeAnalysis)
            {
                Log.LogMessage("Code analysis is disabled.");
                return true;
            }

            logger = new MSBuildLogger(Log);
            Sasa.QualityTools.CodeAnalysis.Common.Logger.Register(logger);
            try
            {
                return PerformCodeAnalysis();
            }
            finally
            {
                Sasa.QualityTools.CodeAnalysis.Common.Logger.Unregister(logger);
            }
        }

        private bool CanExecute()
        {
            if (String.IsNullOrWhiteSpace(TargetPath))
            {
                Log.LogError("The TargetPath is not specified.");
                return false;
            }

            if (String.IsNullOrWhiteSpace(ProjectDir))
            {
                Log.LogError("The ProjectDir is not specified.");
                return false;
            }

            if (References == null)
            {
                Log.LogError("The References is not specified.");
                return false;
            }

            if (String.IsNullOrWhiteSpace(SolutionDir) || SolutionDir == "*Undefined*")
            {
                Log.LogWarning("The SolutionDir is not specified, so {0} probing the directory that contains .sln file.", GetType().Name);
                SolutionDir = ProbeSolutionDir(ProjectDir);
            }
            if (SolutionDir == null)
            {
                Log.LogError("The solution directory can't be found.");
                return false;
            }

            return true;
        }

        private string ProbeSolutionDir(string path)
        {
            if (path == Path.GetPathRoot(path))
            {
                return null;
            }
            else
            {
                var directory = Directory.GetParent(path);
                return (directory.GetFiles("*.sln").Count() > 0) ? directory.FullName : ProbeSolutionDir(directory.FullName);
            }
        }

        protected override string GenerateFullPathToTool()
        {
            throw new NotImplementedException();
        }

        protected override string ToolName
        {
            get { throw new NotImplementedException(); }
        }
    }
}
